@@ -1,29 +1,25 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    This library is SDL_free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_sysaudio.h,v 1.19 2004/01/04 16:49:11 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 #ifndef _SDL_sysaudio_h
 #define _SDL_sysaudio_h
@@ -104,71 +100,83 @@ typedef struct AudioBootStrap {
 	SDL_AudioDevice *(*create)(int devindex);
 } AudioBootStrap;
 
-#ifdef OPENBSD_AUDIO_SUPPORT
-extern AudioBootStrap OPENBSD_AUDIO_bootstrap;
+#if SDL_AUDIO_DRIVER_BSD
+extern AudioBootStrap BSD_AUDIO_bootstrap;
 #endif
-#ifdef OSS_SUPPORT
+#if SDL_AUDIO_DRIVER_PULSE
+extern AudioBootStrap PULSE_bootstrap;
+#endif
+#if SDL_AUDIO_DRIVER_OSS
 extern AudioBootStrap DSP_bootstrap;
 extern AudioBootStrap DMA_bootstrap;
 #endif
-#ifdef ALSA_SUPPORT
+#if SDL_AUDIO_DRIVER_ALSA
 extern AudioBootStrap ALSA_bootstrap;
 #endif
-#ifdef QNXNTOAUDIO_SUPPORT
+#if SDL_AUDIO_DRIVER_QNXNTO
 extern AudioBootStrap QNXNTOAUDIO_bootstrap;
 #endif
-#ifdef SUNAUDIO_SUPPORT
+#if SDL_AUDIO_DRIVER_SUNAUDIO
 extern AudioBootStrap SUNAUDIO_bootstrap;
 #endif
-#ifdef DMEDIA_SUPPORT
+#if SDL_AUDIO_DRIVER_DMEDIA
 extern AudioBootStrap DMEDIA_bootstrap;
 #endif
-#ifdef ARTSC_SUPPORT
-extern AudioBootStrap ARTSC_bootstrap;
+#if SDL_AUDIO_DRIVER_ARTS
+extern AudioBootStrap ARTS_bootstrap;
 #endif
-#ifdef ESD_SUPPORT
+#if SDL_AUDIO_DRIVER_ESD
 extern AudioBootStrap ESD_bootstrap;
 #endif
-#ifdef NAS_SUPPORT
+#if SDL_AUDIO_DRIVER_NAS
 extern AudioBootStrap NAS_bootstrap;
 #endif
-#ifdef ENABLE_DIRECTX
+#if SDL_AUDIO_DRIVER_DSOUND
 extern AudioBootStrap DSOUND_bootstrap;
 #endif
-#ifdef ENABLE_WINDIB
+#if SDL_AUDIO_DRIVER_WAVEOUT
 extern AudioBootStrap WAVEOUT_bootstrap;
 #endif
-#ifdef _AIX
+#if SDL_AUDIO_DRIVER_PAUD
 extern AudioBootStrap Paud_bootstrap;
 #endif
-#ifdef __BEOS__
+#if SDL_AUDIO_DRIVER_BAUDIO
 extern AudioBootStrap BAUDIO_bootstrap;
 #endif
-#if defined(macintosh) || TARGET_API_MAC_CARBON
+#if SDL_AUDIO_DRIVER_COREAUDIO
+extern AudioBootStrap COREAUDIO_bootstrap;
+#endif
+#if SDL_AUDIO_DRIVER_SNDMGR
 extern AudioBootStrap SNDMGR_bootstrap;
 #endif
-#ifdef ENABLE_AHI
-extern AudioBootStrap AHI_bootstrap;
-#endif
-#ifdef MINTAUDIO_SUPPORT
+#if SDL_AUDIO_DRIVER_MINT
 extern AudioBootStrap MINTAUDIO_GSXB_bootstrap;
 extern AudioBootStrap MINTAUDIO_MCSN_bootstrap;
 extern AudioBootStrap MINTAUDIO_STFA_bootstrap;
 extern AudioBootStrap MINTAUDIO_XBIOS_bootstrap;
 extern AudioBootStrap MINTAUDIO_DMA8_bootstrap;
 #endif
-#ifdef DISKAUD_SUPPORT
+#if SDL_AUDIO_DRIVER_DISK
 extern AudioBootStrap DISKAUD_bootstrap;
 #endif
-#ifdef ENABLE_DC
+#if SDL_AUDIO_DRIVER_DUMMY
+extern AudioBootStrap DUMMYAUD_bootstrap;
+#endif
+#if SDL_AUDIO_DRIVER_DC
 extern AudioBootStrap DCAUD_bootstrap;
 #endif
-#ifdef DRENDERER_SUPPORT
-extern AudioBootStrap DRENDERER_bootstrap;
+#if SDL_AUDIO_DRIVER_NDS
+extern AudioBootStrap NDSAUD_bootstrap;
 #endif
-#ifdef MMEAUDIO_SUPPORT
+#if SDL_AUDIO_DRIVER_MMEAUDIO
 extern AudioBootStrap MMEAUDIO_bootstrap;
 #endif
+#if SDL_AUDIO_DRIVER_DART
+extern AudioBootStrap DART_bootstrap;
+#endif
+#if SDL_AUDIO_DRIVER_EPOCAUDIO
+extern AudioBootStrap EPOCAudio_bootstrap; 
+#endif
 
 /* This is the current audio device */
 extern SDL_AudioDevice *current_audio;
