@@ -1,53 +1,42 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
+#include "SDL_config.h"
 
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_blit.c,v 1.8 2003/08/30 00:36:20 slouken Exp $";
-#endif
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "SDL_error.h"
 #include "SDL_video.h"
 #include "SDL_sysvideo.h"
 #include "SDL_blit.h"
 #include "SDL_RLEaccel_c.h"
 #include "SDL_pixels_c.h"
-#include "SDL_memops.h"
 
-#if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
+#if defined(__GNUC__) && (defined(__i386__) || defined(__x86_64__)) && SDL_ASSEMBLY_ROUTINES
+#define MMX_ASMBLIT
+#if (__GNUC__ > 2)  /* SSE instructions aren't in GCC 2. */
+#define SSE_ASMBLIT
+#endif
+#endif
+
+#if defined(MMX_ASMBLIT)
+#include "SDL_cpuinfo.h"
 #include "mmx.h"
-/* Function to check the CPU flags */
-#define MMX_CPU		0x800000
-#define SSE_CPU		0x2000000
-#define CPU_Flags()	Hermes_X86_CPU()
-#define X86_ASSEMBLER
-#define HermesConverterInterface	void
-#define HermesClearInterface		void
-#define STACKCALL
-#include "HeadX86.h"
 #endif
 
 /* The general purpose software blit routine */
@@ -119,8 +108,8 @@ static int SDL_SoftBlit(SDL_Surface *src, SDL_Rect *srcrect,
 	return(okay ? 0 : -1);
 }
 
-#if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
-void SDL_memcpyMMX(char* to,char* from,int len)
+#ifdef MMX_ASMBLIT
+static __inline__ void SDL_memcpyMMX(Uint8 *to, const Uint8 *from, int len)
 {
 	int i;
 
@@ -136,7 +125,8 @@ void SDL_memcpyMMX(char* to,char* from,int len)
 		SDL_memcpy(to, from, len&7);
 }
 
-void SDL_memcpySSE(char* to,char* from,int len)
+#ifdef SSE_ASMBLIT
+static __inline__ void SDL_memcpySSE(Uint8 *to, const Uint8 *from, int len)
 {
 	int i;
 
@@ -160,15 +150,13 @@ void SDL_memcpySSE(char* to,char* from,int len)
 		SDL_memcpy(to, from, len&7);
 }
 #endif
+#endif
 
 static void SDL_BlitCopy(SDL_BlitInfo *info)
 {
 	Uint8 *src, *dst;
 	int w, h;
 	int srcskip, dstskip;
-#if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
-	Uint32 f;
-#endif
 
 	w = info->d_width*info->dst->BytesPerPixel;
 	h = info->d_height;
@@ -176,9 +164,9 @@ static void SDL_BlitCopy(SDL_BlitInfo *info)
 	dst = info->d_pixels;
 	srcskip = w+info->s_skip;
 	dstskip = w+info->d_skip;
-#if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
-	f=CPU_Flags();
-	if((f&(MMX_CPU|SSE_CPU))==(MMX_CPU|SSE_CPU))
+
+#ifdef SSE_ASMBLIT
+	if(SDL_HasSSE())
 	{
 		while ( h-- ) {
 			SDL_memcpySSE(dst, src, w);
@@ -190,7 +178,9 @@ static void SDL_BlitCopy(SDL_BlitInfo *info)
 		::);
 	}
 	else
-	if((f&(MMX_CPU))!=0)
+#endif
+#ifdef MMX_ASMBLIT
+	if(SDL_HasMMX())
 	{
 		while ( h-- ) {
 			SDL_memcpyMMX(dst, src, w);
@@ -288,6 +278,17 @@ int SDL_CalculateBlit(SDL_Surface *surface)
 			video->CheckHWBlit(this, surface, surface->map->dst);
 		}
 	}
+	
+	/* if an alpha pixel format is specified, we can accelerate alpha blits */
+	if (((surface->flags & SDL_HWSURFACE) == SDL_HWSURFACE )&&(current_video->displayformatalphapixel)) 
+	{
+		if ( (surface->flags & SDL_SRCALPHA) ) 
+			if ( current_video->info.blit_hw_A ) {
+				SDL_VideoDevice *video = current_video;
+				SDL_VideoDevice *this  = current_video;
+				video->CheckHWBlit(this, surface, surface->map->dst);
+			}
+	}
 
 	/* Get the blit function index, based on surface mode */
 	/* { 0 = nothing, 1 = colorkey, 2 = alpha, 3 = colorkey+alpha } */
