@@ -1,52 +1,44 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_sysvideo.cc,v 1.7 2002/03/06 11:23:03 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 /* BWindow based framebuffer implementation */
 
-#include <stdlib.h>
-#include <string.h>
-
-#include <stdio.h>
 #include <unistd.h>
 
-#include "SDL.h"
-#include "SDL_BeApp.h"
 #include "SDL_BWin.h"
 #include "SDL_timer.h"
-#include "blank_cursor.h"
 
 extern "C" {
 
-#include "SDL_sysvideo.h"
-#include "SDL_sysmouse_c.h"
+#include "../SDL_sysvideo.h"
+#include "../../events/SDL_events_c.h"
 #include "SDL_sysevents_c.h"
-#include "SDL_events_c.h"
+#include "SDL_sysmouse_c.h"
 #include "SDL_syswm_c.h"
 #include "SDL_lowvideo.h"
+#include "../SDL_yuvfuncs.h"
+#include "SDL_sysyuv.h"
+#include "../blank_cursor.h"
 
 #define BEOS_HIDDEN_SIZE	32	/* starting hidden window size */
 
@@ -65,9 +57,14 @@ static void BE_UnlockHWSurface(_THIS, SDL_Surface *surface);
 static void BE_FreeHWSurface(_THIS, SDL_Surface *surface);
 
 static int BE_ToggleFullScreen(_THIS, int fullscreen);
+static SDL_Overlay *BE_CreateYUVOverlay(_THIS, int width, int height, Uint32 format, SDL_Surface *display);
 
 /* OpenGL functions */
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
+static int BE_GL_LoadLibrary(_THIS, const char *path);
+static void* BE_GL_GetProcAddress(_THIS, const char *proc);
+static int BE_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value);
+static int BE_GL_MakeCurrent(_THIS);
 static void BE_GL_SwapBuffers(_THIS);
 #endif
 
@@ -80,8 +77,8 @@ static int BE_Available(void)
 
 static void BE_DeleteDevice(SDL_VideoDevice *device)
 {
-	free(device->hidden);
-	free(device);
+	SDL_free(device->hidden);
+	SDL_free(device);
 }
 
 static SDL_VideoDevice *BE_CreateDevice(int devindex)
@@ -89,29 +86,33 @@ static SDL_VideoDevice *BE_CreateDevice(int devindex)
 	SDL_VideoDevice *device;
 
 	/* Initialize all variables that we clean on shutdown */
-	device = (SDL_VideoDevice *)malloc(sizeof(SDL_VideoDevice));
+	device = (SDL_VideoDevice *)SDL_malloc(sizeof(SDL_VideoDevice));
 	if ( device ) {
-		memset(device, 0, (sizeof *device));
+		SDL_memset(device, 0, (sizeof *device));
 		device->hidden = (struct SDL_PrivateVideoData *)
-				malloc((sizeof *device->hidden));
+				SDL_malloc((sizeof *device->hidden));
 	}
 	if ( (device == NULL) || (device->hidden == NULL) ) {
 		SDL_OutOfMemory();
 		if ( device ) {
-			free(device);
+			SDL_free(device);
 		}
 		return(0);
 	}
-	memset(device->hidden, 0, (sizeof *device->hidden));
+	SDL_memset(device->hidden, 0, (sizeof *device->hidden));
 
 	/* Set the function pointers */
+	/* Initialization/Query functions */
 	device->VideoInit = BE_VideoInit;
 	device->ListModes = BE_ListModes;
 	device->SetVideoMode = BE_SetVideoMode;
+	device->ToggleFullScreen = BE_ToggleFullScreen;
 	device->UpdateMouse = BE_UpdateMouse;
+	device->CreateYUVOverlay = BE_CreateYUVOverlay;
 	device->SetColors = BE_SetColors;
 	device->UpdateRects = NULL;
 	device->VideoQuit = BE_VideoQuit;
+	/* Hardware acceleration functions */
 	device->AllocHWSurface = BE_AllocHWSurface;
 	device->CheckHWBlit = NULL;
 	device->FillHWRect = NULL;
@@ -121,21 +122,33 @@ static SDL_VideoDevice *BE_CreateDevice(int devindex)
 	device->UnlockHWSurface = BE_UnlockHWSurface;
 	device->FlipHWSurface = NULL;
 	device->FreeHWSurface = BE_FreeHWSurface;
-#ifdef HAVE_OPENGL
+	/* Gamma support */
+#if SDL_VIDEO_OPENGL
+	/* OpenGL support */
+	device->GL_LoadLibrary = BE_GL_LoadLibrary;
+	device->GL_GetProcAddress = BE_GL_GetProcAddress;
+	device->GL_GetAttribute = BE_GL_GetAttribute;
+	device->GL_MakeCurrent = BE_GL_MakeCurrent;
 	device->GL_SwapBuffers = BE_GL_SwapBuffers;
 #endif
-	device->SetIcon = NULL;
+	/* Window manager functions */
 	device->SetCaption = BE_SetWMCaption;
-	device->GetWMInfo = NULL;
+	device->SetIcon = NULL;
+	device->IconifyWindow = BE_IconifyWindow;
+	device->GrabInput = BE_GrabInput;
+	device->GetWMInfo = BE_GetWMInfo;
+	/* Cursor manager functions */
 	device->FreeWMCursor = BE_FreeWMCursor;
 	device->CreateWMCursor = BE_CreateWMCursor;
 	device->ShowWMCursor = BE_ShowWMCursor;
 	device->WarpWMCursor = BE_WarpWMCursor;
+	device->MoveWMCursor = NULL;
+	device->CheckMouseMode = BE_CheckMouseMode;
+	/* Event manager functions */
 	device->InitOSKeymap = BE_InitOSKeymap;
 	device->PumpEvents = BE_PumpEvents;
 
 	device->free = BE_DeleteDevice;
-	device->ToggleFullScreen = BE_ToggleFullScreen;
 
 	/* Set the driver flags */
 	device->handles_any_size = 1;
@@ -215,7 +228,7 @@ static int BE_AddMode(_THIS, int index, unsigned int w, unsigned int h)
 	}
 
 	/* Set up the new video mode rectangle */
-	mode = (SDL_Rect *)malloc(sizeof *mode);
+	mode = (SDL_Rect *)SDL_malloc(sizeof *mode);
 	if ( mode == NULL ) {
 		SDL_OutOfMemory();
 		return(-1);
@@ -231,11 +244,11 @@ static int BE_AddMode(_THIS, int index, unsigned int w, unsigned int h)
 	/* Allocate the new list of modes, and fill in the new mode */
 	next_mode = SDL_nummodes[index];
 	SDL_modelist[index] = (SDL_Rect **)
-	       realloc(SDL_modelist[index], (1+next_mode+1)*sizeof(SDL_Rect *));
+	       SDL_realloc(SDL_modelist[index], (1+next_mode+1)*sizeof(SDL_Rect *));
 	if ( SDL_modelist[index] == NULL ) {
 		SDL_OutOfMemory();
 		SDL_nummodes[index] = 0;
-		free(mode);
+		SDL_free(mode);
 		return(-1);
 	}
 	SDL_modelist[index][next_mode] = mode;
@@ -262,6 +275,8 @@ int BE_VideoInit(_THIS, SDL_PixelFormat *vformat)
 
 	/* Save the current display mode */
 	bscreen.GetMode(&saved_mode);
+	_this->info.current_w = saved_mode.virtual_width;
+	_this->info.current_h = saved_mode.virtual_height;
 
 	/* Determine the screen depth */
 	vformat->BitsPerPixel = ColorSpaceToBitsPerPixel(bscreen.ColorSpace());
@@ -273,7 +288,7 @@ int BE_VideoInit(_THIS, SDL_PixelFormat *vformat)
 
 	/* Get the video modes we can switch to in fullscreen mode */
 	bscreen.GetModeList(&modes, &nmodes);
-	qsort(modes, nmodes, sizeof *modes, CompareModes);
+	SDL_qsort(modes, nmodes, sizeof *modes, CompareModes);
 	for ( i=0; i<nmodes; ++i ) {
 		bpp = ColorSpaceToBitsPerPixel(modes[i].space);
 		//if ( bpp != 0 ) { // There are bugs in changing colorspace
@@ -290,6 +305,12 @@ int BE_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	bounds.bottom = BEOS_HIDDEN_SIZE;
 	SDL_Win = new SDL_BWin(bounds);
 
+#if SDL_VIDEO_OPENGL
+	/* testgl application doesn't load library, just tries to load symbols */
+	/* is it correct? if so we have to load library here */
+	BE_GL_LoadLibrary(_this, NULL);
+#endif
+
 	/* Create the clear cursor */
 	SDL_BlankCursor = BE_CreateWMCursor(_this, blank_cdata, blank_cmask,
 			BLANK_CWIDTH, BLANK_CHEIGHT, BLANK_CHOTX, BLANK_CHOTY);
@@ -330,6 +351,11 @@ static bool BE_FindClosestFSMode(_THIS, int width, int height, int bpp,
 	uint32 i, nmodes;
 	SDL_Rect **modes;
 	display_mode *dmodes;
+	display_mode current;
+	float current_refresh;
+	bscreen.GetMode(&current);
+	current_refresh = (1000 * current.timing.pixel_clock) / 
+	                  (current.timing.h_total * current.timing.v_total);
 
 	modes = SDL_modelist[((bpp+7)/8)-1];
 	for ( i=0; modes[i] && (modes[i]->w > width) &&
@@ -339,8 +365,10 @@ static bool BE_FindClosestFSMode(_THIS, int width, int height, int bpp,
 	if ( ! modes[i] || (modes[i]->w < width) || (modes[i]->h < width) ) {
 		--i;	/* We went too far */
 	}
+
 	width = modes[i]->w;
 	height = modes[i]->h;      
+
 	bscreen.GetModeList(&dmodes, &nmodes);
 	for ( i = 0; i < nmodes; ++i ) {
 		if ( (bpp == ColorSpaceToBitsPerPixel(dmodes[i].space)) &&
@@ -351,6 +379,15 @@ static bool BE_FindClosestFSMode(_THIS, int width, int height, int bpp,
 	}
 	if ( i != nmodes ) {
 		*mode = dmodes[i];
+		if ((mode->virtual_width <= current.virtual_width) &&
+		    (mode->virtual_height <= current.virtual_height)) {
+			float new_refresh = (1000 * mode->timing.pixel_clock) /
+			                    (mode->timing.h_total * mode->timing.v_total);
+			if (new_refresh < current_refresh) {
+				mode->timing.pixel_clock = (uint32)((mode->timing.h_total * mode->timing.v_total)
+				                                    * current_refresh / 1000);
+			}
+		}
 		return true;
 	} else {
 		return false;
@@ -398,7 +435,7 @@ static int BE_SetFullScreen(_THIS, SDL_Surface *screen, int fullscreen)
 	}
 
 	if ( SDL_Win->Lock() ) {
-		int xoff, yoff;
+		int cx, cy;
 		if ( SDL_Win->Shown() ) {
 			needs_unlock = 1;
 			SDL_Win->Hide();
@@ -414,19 +451,18 @@ static int BE_SetFullScreen(_THIS, SDL_Surface *screen, int fullscreen)
 		/* Calculate offsets - used either to center window
 		 * (windowed mode) or to set drawing offsets (fullscreen mode)
 		 */
-		xoff = (bounds.IntegerWidth() - width)/2;
-		yoff = (bounds.IntegerHeight() - height)/2;
+		cx = (bounds.IntegerWidth() - width)/2;
+		cy = (bounds.IntegerHeight() - height)/2;
+
 		if ( fullscreen ) {
 			/* Set offset for drawing */
-			SDL_Win->SetXYOffset(xoff, yoff);
+			SDL_Win->SetXYOffset(cx, cy);
 		} else {
-			/* Center window and reset the drawing offset */
 			SDL_Win->SetXYOffset(0, 0);
 		}
 		if ( ! needs_unlock || was_fullscreen ) {
 			/* Center the window the first time */
-			SDL_Win->MoveTo(xoff > 0 ? (float)xoff : 0.0f,
-					yoff > 0 ? (float)yoff : 0.0f);
+			SDL_Win->MoveTo(cx, cy);
 		}
 		SDL_Win->Show();
 		
@@ -457,9 +493,28 @@ SDL_Surface *BE_SetVideoMode(_THIS, SDL_Surface *current,
 	BScreen bscreen;
 	BBitmap *bbitmap;
 	BRect bounds;
+	Uint32 gl_flags = 0;
 
-	/* Create the view for this window */
-	if ( SDL_Win->CreateView(flags) < 0 ) {
+	/* Only RGB works on r5 currently */
+	gl_flags = BGL_RGB;
+	if (_this->gl_config.double_buffer)
+		gl_flags |= BGL_DOUBLE;
+	else
+		gl_flags |= BGL_SINGLE;
+	if (_this->gl_config.alpha_size > 0 || bpp == 32)
+		gl_flags |= BGL_ALPHA;
+	if (_this->gl_config.depth_size > 0)
+		gl_flags |= BGL_DEPTH;
+	if (_this->gl_config.stencil_size > 0)
+		gl_flags |= BGL_STENCIL;
+	if (_this->gl_config.accum_red_size > 0
+		|| _this->gl_config.accum_green_size > 0
+		|| _this->gl_config.accum_blue_size > 0
+		|| _this->gl_config.accum_alpha_size > 0)
+		gl_flags |= BGL_ACCUM;
+
+	/* Create the view for this window, using found flags */
+	if ( SDL_Win->CreateView(flags, gl_flags) < 0 ) {
 		return(NULL);
 	}
 
@@ -484,7 +539,7 @@ SDL_Surface *BE_SetVideoMode(_THIS, SDL_Surface *current,
 		current->flags |= SDL_OPENGL;
 		current->pitch = 0;
 		current->pixels = NULL;
-		_this->UpdateRects = NULL;		
+		_this->UpdateRects = NULL;
 	} else {
 		/* Create the BBitmap framebuffer */
 		bounds.top = 0; bounds.left = 0;
@@ -572,7 +627,145 @@ static void BE_NormalUpdate(_THIS, int numrects, SDL_Rect *rects)
 	}
 }
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
+/* Passing a NULL path means load pointers from the application */
+int BE_GL_LoadLibrary(_THIS, const char *path)
+{
+	if (path == NULL) {
+		if (_this->gl_config.dll_handle == NULL) {
+			image_info info;
+			int32 cookie = 0;
+			while (get_next_image_info(0,&cookie,&info) == B_OK) {
+				void *location = NULL;
+				if (get_image_symbol((image_id)cookie,"glBegin",B_SYMBOL_TYPE_ANY,&location) == B_OK) {
+					_this->gl_config.dll_handle = (void*)cookie;
+					_this->gl_config.driver_loaded = 1;
+					SDL_strlcpy(_this->gl_config.driver_path, "libGL.so", SDL_arraysize(_this->gl_config.driver_path));
+				}
+			}
+		}
+	} else {
+		/*
+			FIXME None of BeOS libGL.so implementations have exported functions 
+			to load BGLView, which should be reloaded from new lib.
+			So for now just "load" linked libGL.so :(
+		*/
+		if (_this->gl_config.dll_handle == NULL) {
+			return BE_GL_LoadLibrary(_this, NULL);
+		}
+
+		/* Unload old first */
+		/*if (_this->gl_config.dll_handle != NULL) {*/
+			/* Do not try to unload application itself (if LoadLibrary was called before with NULL ;) */
+		/*	image_info info;
+			if (get_image_info((image_id)_this->gl_config.dll_handle, &info) == B_OK) {
+				if (info.type != B_APP_IMAGE) {
+					unload_add_on((image_id)_this->gl_config.dll_handle);
+				}
+			}
+			
+		}
+
+		if ((_this->gl_config.dll_handle = (void*)load_add_on(path)) != (void*)B_ERROR) {
+			_this->gl_config.driver_loaded = 1;
+			SDL_strlcpy(_this->gl_config.driver_path, path, SDL_arraysize(_this->gl_config.driver_path));
+		}*/
+	}
+
+	if (_this->gl_config.dll_handle != NULL) {
+		return 0;
+	} else {
+		_this->gl_config.dll_handle = NULL;
+		_this->gl_config.driver_loaded = 0;
+		*_this->gl_config.driver_path = '\0';
+		return -1;
+	}
+}
+
+void* BE_GL_GetProcAddress(_THIS, const char *proc)
+{
+	if (_this->gl_config.dll_handle != NULL) {
+		void *location = NULL;
+		status_t err;
+		if ((err = get_image_symbol((image_id)_this->gl_config.dll_handle, proc, B_SYMBOL_TYPE_ANY, &location)) == B_OK) {
+			return location;
+		} else {
+			SDL_SetError("Couldn't find OpenGL symbol");
+			return NULL;
+		}
+	} else {
+		SDL_SetError("OpenGL library not loaded");
+		return NULL;
+	}
+}
+
+int BE_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value)
+{
+	/*
+		FIXME? Right now BE_GL_GetAttribute shouldn't be called between glBegin() and glEnd() - it doesn't use "cached" values
+	*/
+	switch (attrib)
+    {
+		case SDL_GL_RED_SIZE:
+			glGetIntegerv(GL_RED_BITS, (GLint*)value);
+			break;
+		case SDL_GL_GREEN_SIZE:
+			glGetIntegerv(GL_GREEN_BITS, (GLint*)value);
+			break;
+		case SDL_GL_BLUE_SIZE:
+			glGetIntegerv(GL_BLUE_BITS, (GLint*)value);
+			break;
+		case SDL_GL_ALPHA_SIZE:
+			glGetIntegerv(GL_ALPHA_BITS, (GLint*)value);
+			break;
+		case SDL_GL_DOUBLEBUFFER:
+			glGetBooleanv(GL_DOUBLEBUFFER, (GLboolean*)value);
+			break;
+		case SDL_GL_BUFFER_SIZE:
+			int v;
+			glGetIntegerv(GL_RED_BITS, (GLint*)&v);
+			*value = v;
+			glGetIntegerv(GL_GREEN_BITS, (GLint*)&v);
+			*value += v;
+			glGetIntegerv(GL_BLUE_BITS, (GLint*)&v);
+			*value += v;
+			glGetIntegerv(GL_ALPHA_BITS, (GLint*)&v);
+			*value += v;
+			break;
+		case SDL_GL_DEPTH_SIZE:
+			glGetIntegerv(GL_DEPTH_BITS, (GLint*)value); /* Mesa creates 16 only? r5 always 32 */
+			break;
+		case SDL_GL_STENCIL_SIZE:
+			glGetIntegerv(GL_STENCIL_BITS, (GLint*)value);
+			break;
+		case SDL_GL_ACCUM_RED_SIZE:
+			glGetIntegerv(GL_ACCUM_RED_BITS, (GLint*)value);
+			break;
+		case SDL_GL_ACCUM_GREEN_SIZE:
+			glGetIntegerv(GL_ACCUM_GREEN_BITS, (GLint*)value);
+			break;
+		case SDL_GL_ACCUM_BLUE_SIZE:
+			glGetIntegerv(GL_ACCUM_BLUE_BITS, (GLint*)value);
+			break;
+		case SDL_GL_ACCUM_ALPHA_SIZE:
+			glGetIntegerv(GL_ACCUM_ALPHA_BITS, (GLint*)value);
+			break;
+		case SDL_GL_STEREO:
+		case SDL_GL_MULTISAMPLEBUFFERS:
+		case SDL_GL_MULTISAMPLESAMPLES:
+		default:
+			*value=0;
+			return(-1);
+	}
+	return 0;
+}
+
+int BE_GL_MakeCurrent(_THIS)
+{
+	/* FIXME: should we glview->unlock and then glview->lock()? */
+	return 0;
+}
+
 void BE_GL_SwapBuffers(_THIS)
 {
 	SDL_Win->SwapBuffers();
@@ -600,6 +793,9 @@ void BE_VideoQuit(_THIS)
 {
 	int i, j;
 
+	SDL_Win->Quit();
+	SDL_Win = NULL;
+
 	if ( SDL_BlankCursor != NULL ) {
 		BE_FreeWMCursor(_this, SDL_BlankCursor);
 		SDL_BlankCursor = NULL;
@@ -607,9 +803,9 @@ void BE_VideoQuit(_THIS)
 	for ( i=0; i<NUM_MODELISTS; ++i ) {
 		if ( SDL_modelist[i] ) {
 			for ( j=0; SDL_modelist[i][j]; ++j ) {
-				free(SDL_modelist[i][j]);
+				SDL_free(SDL_modelist[i][j]);
 			}
-			free(SDL_modelist[i]);
+			SDL_free(SDL_modelist[i]);
 			SDL_modelist[i] = NULL;
 		}
 	}
@@ -621,6 +817,12 @@ void BE_VideoQuit(_THIS)
 		}
 		_this->screen->pixels = NULL;
 	}
+
+#if SDL_VIDEO_OPENGL
+	if (_this->gl_config.dll_handle != NULL)
+		unload_add_on((image_id)_this->gl_config.dll_handle);
+#endif
+
 	SDL_QuitBeApp();
 }
 
